@@ -22,7 +22,10 @@ import sendSMS
 import locale
 import dbus
 import pango
+import xml.dom.minidom
+import xml.parsers.expat
 from portrait import FremantleRotation
+from virtualKeyboard import vKeyboard
 
 # By using apt-cache search, the following packages are required:
 # python2.5-gtk2, python2.5-hildon, python-gobject
@@ -48,7 +51,11 @@ class StrokeOrder():
 	textArea = hildon.TextView()	# Define the text area for input
 	textArea.set_placeholder(_("StrokeOrder_textAreaPlaceHolder"))	# Set the info text for display when text area is empty
 	textArea.set_wrap_mode(gtk.WRAP_CHAR)		
-	textArea.set_cursor_visible(True)
+	textArea.set_cursor_visible(True)	
+	
+	vkb = vKeyboard(textArea)
+	vkbVBox = vkb.getKeyboard()
+	vnpVBox = vkb.getNumpad()
 	
 	charLeft = gtk.Label()	# Define label for displaying number of characters left and number of messages being sent
 	charLeft.set_text("70(1)")
@@ -60,13 +67,13 @@ class StrokeOrder():
 
 	sendToNumbers = ""	# Variable for remembering the contact numbers when user switches back and forth the pages
 	
-	con=sqlite3.connect("/opt/StrokeOrder/IM.db")	# Connect to stroke lookup database
-	con.text_factory=str
-	cur = con.cursor()			
-	
 	osso_ctx = osso.Context("test_abook", "0.one")	# Define variable for Address Book
 	capi = pygobject.PyGObjectCPAI()
-
+	
+	vKB = False
+	vNP = False
+	orientation = ""
+	
 	def __init__(self):	
 		self.main()
 			
@@ -78,17 +85,70 @@ class StrokeOrder():
 			self.resizeControls(480,744)
 		else:
 			self.resizeControls(800,424)
-		
+
+	# This method returns a default setting XML string
+	def getDefaultSettingsXML(self):
+		defaultSettingsXML = "<?xml version=\"1.0\"?><Settings><UseVirtualKB>" + str(True) + "</UseVirtualKB><UseCustIMDB>" + str(True) + "</UseCustIMDB><CandCustOrder>" + str(True) +"</CandCustOrder><AssocCustOrder>" + str(True) + "</AssocCustOrder></Settings>" 
+		return defaultSettingsXML
+
+	# This method writes the default settings to the actual XML file
+	def writeDefaultSettingsXML(self):
+		self.settingsDOM = xml.dom.minidom.parseString(self.getDefaultSettingsXML())
+		self.settingsFile = open("/home/user/MyDocs/.StrokeOrder/settings.xml", "w")
+		self.settingsFile.write(self.settingsDOM.toxml("UTF-8"))
+		self.settingsFile.close()
+
+	# This method writes the currently selected settings to the actual XML file 
+	def writeSettingsXML(self,UseVirtualKB,UseCustIMDB,CandCustOrder,AssocCustOrder):
+		self.SettingsXML = "<?xml version=\"1.0\"?><Settings><UseVirtualKB>" + str(UseVirtualKB) + "</UseVirtualKB><UseCustIMDB>" + str(UseCustIMDB) + "</UseCustIMDB><CandCustOrder>" + str(CandCustOrder) +"</CandCustOrder><AssocCustOrder>" + str(AssocCustOrder) + "</AssocCustOrder></Settings>" 
+		self.settingsFile = open("/home/user/MyDocs/.StrokeOrder/settings.xml", "w")
+		self.settingsFile.write(self.SettingsXML)
+		self.settingsFile.close() 
+
+	# This method takes in the setting XML DOM object and the name of a node, and then returns the value of that node.
+	def getKeyValue(self, settingsDOM, key):
+		returnValue = ""
+		for node in self.settingsDOM.getElementsByTagName(key)[0].childNodes:
+			if node.nodeType == node.TEXT_NODE:
+				returnValue = returnValue + node.data
+		return returnValue
+
+	# This method creates a directory for the settings file under MyDocs/ , or returns true if it is already created
+	def makedir(self,dirname):
+		if os.path.isdir(dirname):
+			return True
+		elif os.path.isfile(dirname):
+			return False
+		else:
+			head, tail = os.path.split(dirname)
+			if head and not os.path.isdir(head):
+				self.makedir(head)
+			if tail:
+				os.mkdir(dirname)
+		return True
+  
 	def resizeControls(self, AppWidth, AppHeight):
 		self.eb.set_size_request(AppWidth,50)
-		self.buttonHBox1.set_size_request(AppWidth,int(AppHeight*0.165))
-		self.buttonHBox2.set_size_request(AppWidth,int(AppHeight*0.165))
+		if self.vKB == False and self.vNP == False:
+			self.buttonHBox1.set_size_request(AppWidth,int(AppHeight*0.165))
+			self.buttonHBox2.set_size_request(AppWidth,int(AppHeight*0.165))		
+			self.EngButton.set_size_request(int(AppWidth/3),int(AppHeight*0.165))
+			self.SymButton.set_size_request(int(AppWidth/3),int(AppHeight*0.165))
+			self.SMSPageButton.set_size_request(int(AppWidth/3),int(AppHeight*0.165))
+			self.textAreaButtonsHBox.set_size_request(AppWidth,int(AppHeight*0.165))
+		elif self.vKB == True and self.vNP == False:
+			self.EngButton.set_size_request(int(AppWidth/3),int(AppHeight*0.1))
+			self.SymButton.set_size_request(int(AppWidth/3),int(AppHeight*0.1))
+			self.SMSPageButton.set_size_request(int(AppWidth/3),int(AppHeight*0.1))
+			self.textAreaButtonsHBox.set_size_request(AppWidth,int(AppHeight*0.1))
+			self.vkbVBox.set_size_request(AppWidth,int(AppHeight*0.45))
+		elif self.vKB == False and self.vNP == True:
+			self.EngButton.set_size_request(int(AppWidth/3),int(AppHeight*0.1))
+			self.SymButton.set_size_request(int(AppWidth/3),int(AppHeight*0.1))
+			self.SMSPageButton.set_size_request(int(AppWidth/3),int(AppHeight*0.1))
+			self.textAreaButtonsHBox.set_size_request(AppWidth,int(AppHeight*0.1))
+			self.vnpVBox.set_size_request(AppWidth,int(AppHeight*0.45))
 		
-		self.EngButton.set_size_request(int(AppWidth/3),int(AppHeight*0.165))
-		self.SymButton.set_size_request(int(AppWidth/3),int(AppHeight*0.165))
-		self.SMSPageButton.set_size_request(int(AppWidth/3),int(AppHeight*0.165))
-		
-		self.textAreaButtonsHBox.set_size_request(AppWidth,int(AppHeight*0.165))
 		self.candidateChars.set_size_request(150,int(AppHeight*0.2830))
 		self.candidateCharsAndtextAreaHBox.set_size_request(int(AppWidth),int(AppHeight*0.2830))			
 	
@@ -131,9 +191,9 @@ class StrokeOrder():
 		# Takes in a Chinese character and display its associated phrases for selection
 		if charKey!="":
 			if self.getCurrentLocale() == "zh_HK" or self.getCurrentLocale() == "zh_Yue" or self.getCurrentLocale() == "zh_TW":	# Display Traditional Chinese associated phrases first if system locale is Chinese (Hong Kong), Cantonese or Chinese (Taiwan)
-				self.cur.execute("SELECT DISTINCT Phrase FROM Assoc WHERE CharKey='" + charKey + "' ORDER BY TradSim DESC, Phrase ASC")		
+				self.cur.execute("SELECT DISTINCT Phrase FROM Assoc WHERE CharKey='" + charKey + "' ORDER BY Frequency DESC, TradSim DESC, Phrase ASC")		
 			else: # Display Simplifeid Chinese associated phrases first if system locale is Chinese (PRC), Chinese, or English
-				self.cur.execute("SELECT DISTINCT Phrase FROM Assoc WHERE CharKey='" + charKey + "' ORDER BY TradSim ASC, Phrase ASC")					
+				self.cur.execute("SELECT DISTINCT Phrase FROM Assoc WHERE CharKey='" + charKey + "' ORDER BY Frequency DESC, TradSim ASC, Phrase ASC")
 			self.con.commit()
 		
 		tm = gtk.ListStore(gobject.TYPE_STRING)
@@ -143,7 +203,40 @@ class StrokeOrder():
 			for row in self.cur.fetchall():		
 				self.candidateChars.append_text(row[0])	# Generate associated phrases selection list
 		
+	def connectIMDB(self): # Connect to user's customized database or system's default database, based on the value stored in the settings file
+		try:
+			if self.makedir('/home/user/MyDocs/.StrokeOrder') == True:
+				self.settingsDOM = xml.dom.minidom.parse('/home/user/MyDocs/.StrokeOrder/settings.xml') # parse the settings XML and returns the XML DOM object	
+		except xml.parsers.expat.ExpatError: # displays error message in case of parse error
+				self.writeDefaultSettingsXML()
+				self.settingsDOM = xml.dom.minidom.parseString(self.getDefaultSettingsXML())
+		except IOError: # displays error message in case of file I/O error
+				self.writeDefaultSettingsXML()
+				self.settingsDOM = xml.dom.minidom.parseString(self.getDefaultSettingsXML())
+				
+		if self.getKeyValue(self.settingsDOM,"UseCustIMDB") == "True":
+			if os.path.exists("/home/user/MyDocs/.StrokeOrder/IM.db") == False:
+				try:
+					if self.makedir('/home/user/MyDocs/.StrokeOrder') == True:	# Copy default IM.db to user's folder if it doesn't exist
+						copycmd = "cp /opt/StrokeOrder/IM.db /home/user/MyDocs/.StrokeOrder/"
+						os.system(copycmd)
+						print "IM.db copied to /home/user/MyDocs/.StrokeOrder/"
+				except IOError: # displays error message in case of file I/O error
+					dialog = hildon.Note(hildon.NOTE_TYPE_INFORMATION_THEME, window, _("StrokeOrder_ResetIMDBFailed")) 
+					response = dialog.run()
+					dialog.destroy()										
+			self.con=sqlite3.connect("/home/user/MyDocs/.StrokeOrder/IM.db")	# Connect to the customized stroke lookup database			
+			print "User's customized IM.db is being used."
+		else:
+			self.con=sqlite3.connect("/opt/StrokeOrder/IM.db")	# Connect to default stroke lookup database	
+			print "System's default IM.db is being used."
+			
+		self.con.text_factory=str
+		self.cur = self.con.cursor()			
+		
 	def main(self):	
+		self.connectIMDB()		
+				
 		system_bus = dbus.Bus.get_system()
 		system_bus.add_signal_receiver(self._on_orientation_signal, \
 			signal_name='sig_device_orientation_ind', \
@@ -180,7 +273,7 @@ class StrokeOrder():
 		self.textAreaButtonsVBox = gtk.VBox(False, 0)	
 		self.textAreaButtonsHBox = gtk.HBox(False, 0)	
 		 
-		vbox = gtk.VBox(False, 0)
+		self.mainVBox = gtk.VBox(False, 0)
 		
 		OneButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
 		OneButton.set_label("一")   
@@ -201,14 +294,21 @@ class StrokeOrder():
 		FiveButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
 		FiveButton.set_label("乛")   
 		FiveButton.connect("clicked", self.StrokeClicked, 5)
+		
+		if self.getKeyValue(self.settingsDOM,"UseVirtualKB") == "True":
+			self.EngButton = gtk.ToggleButton(_("StrokeOrder_EngMode"))
+			self.EngButton.connect("clicked", self.showHideVirtualKeyboard, window)
+			self.SymButton = gtk.ToggleButton(_("StrokeOrder_SymMode"))
+			self.SymButton.connect("clicked", self.showHideVirtualNumpad, window)
+		else:
+			self.EngButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+			self.EngButton.set_label(_("StrokeOrder_EngMode"))   
+			self.EngButton.connect("clicked", self.StrokeClicked, 6)
+			
+			self.SymButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+			self.SymButton.set_label(_("StrokeOrder_SymMode"))   
+			self.SymButton.connect("clicked", self.StrokeClicked, 7)	
 
-		self.EngButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
-		self.EngButton.set_label(_("StrokeOrder_EngMode"))   
-		self.EngButton.connect("clicked", self.StrokeClicked, 6)
-
-		self.SymButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
-		self.SymButton.set_label(_("StrokeOrder_SymMode"))   
-		self.SymButton.connect("clicked", self.StrokeClicked, 7)
 		
 		WildcardButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
 		WildcardButton.set_label("？")   
@@ -263,23 +363,63 @@ class StrokeOrder():
 		self.queryString(self.strokesCodeInputted)
 		self.strokesInputted.set_text( self.ConvertCodeToChar(self.strokesCodeInputted) )	# Display the inputted strokes
 		
-		vbox.pack_start(self.eb, False, False ,0)				
-		vbox.pack_start(self.candidateCharsAndtextAreaHBox, True, True, 0)		
-		vbox.pack_start(self.buttonHBox1, False, True ,0)
-		vbox.pack_start(self.buttonHBox2, False, True ,0)		
-		vbox.pack_start(self.textAreaButtonsHBox, False, True, 0)		
-		
+		self.mainVBox.pack_start(self.eb, False, False ,0)				
+		self.mainVBox.pack_start(self.candidateCharsAndtextAreaHBox, True, True, 0)		
+		self.mainVBox.pack_start(self.buttonHBox1, False, True ,0)
+		self.mainVBox.pack_start(self.buttonHBox2, False, True ,0)		
+		self.mainVBox.pack_start(self.textAreaButtonsHBox, False, True, 0)				
 		self.resizeControls(800,424)
 		
 		window.set_title(_("StrokeOrder_AppName")) 
 		window.connect("delete_event",self.app_preclose)
 		menu = self.create_menu(window, "MainPage") 
 		window.set_app_menu(menu)	
-		window.add(vbox)	
+		window.add(self.mainVBox)	
 		window.show_all()		
 		self.textArea.grab_focus()
 		gtk.main()			
 		
+	def showHideVirtualKeyboard(self, EngButton, window):			
+		if self.EngButton.get_active() == False:
+			self.vKB = False			
+			self.mainVBox.remove(self.vkbVBox)
+		else: 
+			if self.SymButton.get_active() == True:
+				self.SymButton.set_active(False)
+				self.showHideVirtualNumpad(self.SymButton,window)
+				
+			self.buttonHBox1.set_size_request(0,0)
+			self.buttonHBox2.set_size_request(0,0)
+			self.mainVBox.pack_start(self.vkbVBox,False,True,0)
+			self.vKB = True
+			self.StrokeClicked(self.EngButton,6)
+			window.show_all()	
+
+		if self.orientation == "portrait":
+			self.resizeControls(480,744)
+		else:
+			self.resizeControls(800,424)			
+			
+	def showHideVirtualNumpad(self, SymButton, window):			
+		if self.SymButton.get_active() == False:
+			self.vNP = False
+			self.mainVBox.remove(self.vnpVBox)
+		else: 
+			if self.EngButton.get_active() == True:
+				self.EngButton.set_active(False)
+				self.showHideVirtualKeyboard(self.EngButton,window)
+			self.buttonHBox1.set_size_request(0,0)
+			self.buttonHBox2.set_size_request(0,0)
+			self.mainVBox.pack_start(self.vnpVBox,False,True,0)
+			self.vNP = True
+			self.StrokeClicked(self.SymButton,7)
+			window.show_all()				
+
+		if self.orientation == "portrait":
+			self.resizeControls(480,744)
+		else:
+			self.resizeControls(800,424)			
+			
 	def aboutProduct(self, AboutProductButton, window):
 		# Show the About this Product dialog box
 		
@@ -288,6 +428,159 @@ class StrokeOrder():
 		response = dialog.run()
 		dialog.destroy()
 
+	def settingsClicked(self, QueryStrokesButton, window):
+		# Create the UI for the Settings page
+		settingsPage = hildon.StackableWindow()
+		settingsPage.set_title(_("StrokeOrder_AppName") + " - " + _("StrokeOrder_Settings"))
+		
+		try:
+			if self.makedir('/home/user/MyDocs/.StrokeOrder') == True:
+				self.settingsDOM = xml.dom.minidom.parse('/home/user/MyDocs/.StrokeOrder/settings.xml') # parse the settings XML and returns the XML DOM object	
+		except xml.parsers.expat.ExpatError: # displays error message in case of parse error
+				self.writeDefaultSettingsXML()
+				self.settingsDOM = xml.dom.minidom.parseString(self.getDefaultSettingsXML())
+		except IOError: # displays error message in case of file I/O error
+				self.writeDefaultSettingsXML()
+				self.settingsDOM = xml.dom.minidom.parseString(self.getDefaultSettingsXML())
+
+		UseVirtualKBLabel = gtk.Label()
+		UseVirtualKBLabel.set_text(_("StrokeOrder_UseVirtualKBLabel"))
+		
+		if self.getKeyValue(self.settingsDOM,"UseVirtualKB") == "True":
+			self.UseVirtualKBButton = gtk.ToggleButton(_("StrokeOrder_UseVirtualKB_On"))
+			self.UseVirtualKBButton.set_active(True)
+		else:
+			self.UseVirtualKBButton = gtk.ToggleButton(_("StrokeOrder_UseVirtualKB_Off"))
+			self.UseVirtualKBButton.set_active(False)
+			
+		self.UseVirtualKBButton.connect("clicked",self.UseVirtualKBButtonClicked, window)		
+
+		UseVirtualKBHBox = gtk.HBox()		
+		UseVirtualKBHBox.pack_start(UseVirtualKBLabel,True,True,0)
+		UseVirtualKBHBox.pack_start(self.UseVirtualKBButton,True,True,0)
+		
+		CandCustOrderLabel = gtk.Label()
+		CandCustOrderLabel.set_text(_("StrokeOrder_CandCustOrderLabel"))
+		
+		if self.getKeyValue(self.settingsDOM,"CandCustOrder") == "True":
+			self.CandCustOrderButton = gtk.ToggleButton(_("StrokeOrder_CandCustOrder_On"))
+			self.CandCustOrderButton.set_active(True)
+		else:
+			self.CandCustOrderButton = gtk.ToggleButton(_("StrokeOrder_CandCustOrder_Off"))
+			self.CandCustOrderButton.set_active(False)
+			
+		self.CandCustOrderButton.connect("clicked",self.CandCustOrderButtonClicked)		
+
+		CandCustOrderHBox = gtk.HBox()		
+		CandCustOrderHBox.pack_start(CandCustOrderLabel,True,True,0)
+		CandCustOrderHBox.pack_start(self.CandCustOrderButton,True,True,0)
+		
+		AssocCustOrderLabel = gtk.Label()
+		AssocCustOrderLabel.set_text(_("StrokeOrder_AssocCustOrderLabel"))
+		
+		if self.getKeyValue(self.settingsDOM,"AssocCustOrder") == "True":
+			self.AssocCustOrderButton = gtk.ToggleButton(_("StrokeOrder_AssocCustOrder_On"))
+			self.AssocCustOrderButton.set_active(True)
+		else:
+			self.AssocCustOrderButton = gtk.ToggleButton(_("StrokeOrder_AssocCustOrder_Off"))
+			self.AssocCustOrderButton.set_active(False)
+			
+		self.AssocCustOrderButton.connect("clicked",self.AssocCustOrderButtonClicked)		
+		
+		AssocCustOrderHBox = gtk.HBox()		
+		AssocCustOrderHBox.pack_start(AssocCustOrderLabel,True,True,0)
+		AssocCustOrderHBox.pack_start(self.AssocCustOrderButton,True,True,0)
+
+		UseCustIMDBLabel = gtk.Label()
+		UseCustIMDBLabel.set_text(_("StrokeOrder_UseCustIMDBLabel"))
+		
+		if self.getKeyValue(self.settingsDOM,"UseCustIMDB") == "True":
+			self.UseCustIMDBButton = gtk.ToggleButton(_("StrokeOrder_UseCustIMDB_On"))
+			self.UseCustIMDBButton.set_active(True)			
+			self.CandCustOrderButton.set_sensitive(True)
+			self.AssocCustOrderButton.set_sensitive(True)			
+		else:
+			self.UseCustIMDBButton = gtk.ToggleButton(_("StrokeOrder_UseCustIMDB_Off"))
+			self.UseCustIMDBButton.set_active(False)
+			self.CandCustOrderButton.set_sensitive(False)
+			self.AssocCustOrderButton.set_sensitive(False)			
+			
+		self.UseCustIMDBButton.connect("clicked",self.UseCustIMDBButtonClicked)		
+		
+		UseCustIMDBHBox = gtk.HBox()		
+		UseCustIMDBHBox.pack_start(UseCustIMDBLabel,True,True,0)
+		UseCustIMDBHBox.pack_start(self.UseCustIMDBButton,True,True,0)			
+		self.ResetIMDBButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+		self.ResetIMDBButton.set_label(_("StrokeOrder_ResetIMDB"))   
+		self.ResetIMDBButton.connect("clicked", self.ResetIMDBButtonClicked, window)
+		
+		vbox = gtk.VBox()
+		vbox.pack_start(UseVirtualKBHBox, True, True, 0)								
+		vbox.pack_start(UseCustIMDBHBox, True, True, 0)								
+		vbox.pack_start(CandCustOrderHBox, True, True, 0)								
+		vbox.pack_start(AssocCustOrderHBox, True, True, 0)								
+		vbox.pack_start(self.ResetIMDBButton, True, True, 0)
+		settingsPage.connect("delete_event",self.settingsPageClosed)
+		settingsPage.add(vbox)
+		settingsPage.show_all()					
+
+	def UseVirtualKBButtonClicked(self, UseVirtualKBButton, window):
+		if self.UseVirtualKBButton.get_active() == True:		
+			self.UseVirtualKBButton.set_label(_("StrokeOrder_UseVirtualKB_On"))
+		else:
+			self.UseVirtualKBButton.set_label(_("StrokeOrder_UseVirtualKB_Off"))
+		dialog = hildon.Note(hildon.NOTE_TYPE_INFORMATION_THEME, window, _("StrokeOrder_UseVirtualKBChange")) 
+		response = dialog.run()
+		dialog.destroy()					
+			
+	def UseCustIMDBButtonClicked(self, UseCustIMDBButton):
+		if self.UseCustIMDBButton.get_active() == True:
+			self.UseCustIMDBButton.set_label(_("StrokeOrder_UseCustIMDB_On"))
+			self.CandCustOrderButton.set_sensitive(True)
+			self.AssocCustOrderButton.set_sensitive(True)
+		else:
+			self.UseCustIMDBButton.set_label(_("StrokeOrder_UseCustIMDB_Off"))
+			self.CandCustOrderButton.set_sensitive(False)
+			self.AssocCustOrderButton.set_sensitive(False)
+			
+	def CandCustOrderButtonClicked(self, CandCustOrderButton):
+		if self.CandCustOrderButton.get_active() == True:
+			self.CandCustOrderButton.set_label(_("StrokeOrder_CandCustOrder_On"))
+		else:
+			self.CandCustOrderButton.set_label(_("StrokeOrder_CandCustOrder_Off"))
+
+	def AssocCustOrderButtonClicked(self, AssocCustOrderButton):
+		if self.AssocCustOrderButton.get_active() == True:
+			self.AssocCustOrderButton.set_label(_("StrokeOrder_AssocCustOrder_On"))
+		else:
+			self.AssocCustOrderButton.set_label(_("StrokeOrder_AssocCustOrder_Off"))
+
+	def ResetIMDBButtonClicked(self, ResetIMDBButton, window):
+		confirm = hildon.hildon_note_new_confirmation(window, _("StrokeOrder_Confirm_ResetIMDB"))
+		response = gtk.Dialog.run(confirm)
+		gtk.Dialog.hide(confirm)
+		if response == gtk.RESPONSE_DELETE_EVENT:
+			print "Reset IM.db cancelled"
+		elif response == gtk.RESPONSE_OK:	
+			try:
+				if self.makedir('/home/user/MyDocs/.StrokeOrder') == True:
+					copycmd = "cp /opt/StrokeOrder/IM.db /home/user/MyDocs/.StrokeOrder/"
+					os.system(copycmd)						
+			except IOError: # displays error message in case of file I/O error
+				dialog = hildon.Note(hildon.NOTE_TYPE_INFORMATION_THEME, window, _("StrokeOrder_ResetIMDBFailed")) 
+				response = dialog.run()
+				dialog.destroy()			
+			
+			dialog = hildon.Note(hildon.NOTE_TYPE_INFORMATION_THEME, window, _("StrokeOrder_ResetIMDBSuccess")) 
+			response = dialog.run()
+			dialog.destroy()			
+		
+	def settingsPageClosed(self, window, widget):
+		self.writeSettingsXML(self.UseVirtualKBButton.get_active(),self.UseCustIMDBButton.get_active(),self.CandCustOrderButton.get_active(),self.AssocCustOrderButton.get_active())
+		self.connectIMDB()
+		# Set the input focus back to the text area once the Settings page is closed		
+		self.textArea.grab_focus()
+		
 	def queryStrokesClicked(self, QueryStrokesButton, window):
 		# Create the UI for the stroke query module
 		
@@ -569,6 +862,10 @@ class StrokeOrder():
 		QueryStrokesButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
 		QueryStrokesButton.set_label(_("StrokeOrder_QueryStrokes"))   
 		QueryStrokesButton.connect("clicked", self.queryStrokesClicked, window)			
+
+		SettingsButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+		SettingsButton.set_label(_("StrokeOrder_Settings"))
+		SettingsButton.connect("clicked", self.settingsClicked, window)			
 		
 		if page == "MainPage":
 			NewButton.set_sensitive(True)
@@ -577,7 +874,8 @@ class StrokeOrder():
 		menu.append(NewButton)							
 		menu.append(QueryStrokesButton)
 		menu.append(CopyButton)
-		menu.append(PasteButton)		
+		menu.append(PasteButton)
+		menu.append(SettingsButton)
 		menu.append(AboutButton)   		
 
 		menu.show_all()
@@ -595,9 +893,21 @@ class StrokeOrder():
 		s = s.replace("6",_("StrokeOrder_EngModeFull"))
 		s = s.replace("7",_("StrokeOrder_SymModeFull"))
 		s = s.replace("%","？")
-
 		return s
 
+	def ConvertCharToCode(self, strokesInChar):
+		# Convert actual strokes to stroke input code in numbers
+		s = strokesInChar
+		s = s.replace("一","1")
+		s = s.replace("丨","2")
+		s = s.replace("丿","3")
+		s = s.replace("丶","4")
+		s = s.replace("乛","5")
+		s = s.replace(_("StrokeOrder_EngModeFull"),"6")
+		s = s.replace(_("StrokeOrder_SymModeFull"),"7")
+		s = s.replace("？","%")
+		return s
+		
 	def setCurrentTextboxInFocus(self, widget, textbox):
 		# Set the input focus to the given text field
 		self.textboxInFocus = textbox
@@ -661,8 +971,22 @@ class StrokeOrder():
 				
 		if not "6" in self.strokesCodeInputted and not "7" in self.strokesCodeInputted:	# If any of the stroke 1-5 is pressed, once text is inserted into the text area, clear the stroke sequence variable and associated phrases
 			text = unicode(text, "utf-8")		
-			if len(text)>1:				
+		
+			if self.strokesInputted.get_text().find(_("StrokeOrder_SelectAssoc1")) > -1 and self.strokesInputted.get_text().find(_("StrokeOrder_SelectAssoc2")) > -1:
+				if self.getKeyValue(self.settingsDOM,"UseCustIMDB") == "True" and self.getKeyValue(self.settingsDOM,"AssocCustOrder") == "True":	# If selected phrase was from an associated phrase...
+					OriginalCharKey = self.strokesInputted.get_text().replace(_("StrokeOrder_SelectAssoc1"),"").replace(_("StrokeOrder_SelectAssoc2"),"") # Obtain the original Character Key
+					self.cur.execute("UPDATE Assoc SET Frequency=(SELECT Frequency FROM Assoc WHERE CharKey='" + OriginalCharKey + "' AND Phrase='" + text + "' LIMIT 1)+1 WHERE CharKey='" + OriginalCharKey + "' AND Phrase='" + text + "';")	# Increment the frequency by 1 so that next time the same phrase can be shown earlier
+					self.con.commit()
+					print "Frequency of Associated Phrase CharKey=" + OriginalCharKey + " and text = " + text + " is incremented by 1."
+			else:	# type strokes and then select single character
+				if self.getKeyValue(self.settingsDOM,"UseCustIMDB") == "True" and self.getKeyValue(self.settingsDOM,"CandCustOrder") == "True":
+					OriginalStrokes = self.ConvertCharToCode(self.strokesInputted.get_text())
+					self.cur.execute("UPDATE StrokeOrder SET Frequency=(SELECT Frequency FROM StrokeOrder WHERE Strokes LIKE '" + OriginalStrokes + "%' AND Character='" + text + "' LIMIT 1)+1 WHERE Strokes LIKE '" + OriginalStrokes + "%' AND Character='" + text + "';")	# Increment the frequency by 1 so that next time the same character can be shown earlier
+					self.con.commit()					
+					print "Frequency of character " + text + " with Strokes beginning with " + OriginalStrokes + " is incremented by 1."
+			if len(text)>1:	# Use the last character of the phrase if the length of the selected phrase is > 1
 				text = text[-1]
+				
 			self.strokesInputted.set_text(_("StrokeOrder_SelectAssoc1") + text + _("StrokeOrder_SelectAssoc2"))	# Clear the stroke display area once the text is inserted
 			self.strokesCodeInputted = ""
 			self.queryString("")			
